==> dart_chainsql_api-1.0.1.tar.gz: lib/src/utils/Util.dart <==
// Copyright (c) 2013-present, the authors of the Pointy Castle project
// This library is dually licensed under LGPL 3 and MPL 2.0.
// See file LICENSE for more information.

//library pointycastle.test.test.src.helpers;

import "dart:typed_data";
import 'dart:math';

import 'package:crypto/crypto.dart';

import "package:base58check/base58.dart";
import "package:hex/hex.dart";

class Util {
  static const String rippleAlphabet =
      "zpxhncf39wBUDNEGHJKLM4PQRST7VWXYZ2badeCg65jkm8oFqi1tuvAsyr";

  /**
      / * @param      src      the source array.
   * @param      srcPos   starting position in the source array.
   * @param      dest     the destination array.
   * @param      destPos  starting position in the destination data.
   * @param      length   the number of array elements to be copied.
   * @exception  IndexOutOfBoundsException  if copying would cause
   *               access of data outside array bounds.
   * @exception  ArrayStoreException  if an element in the <code>src</code>
   *               array could not be stored into the <code>dest</code> array
   *               because of a type mismatch.
   * @exception  NullPointerException if either <code>src</code> or
   *               <code>dest</code> is <code>null</code>.
   */
  static arraycopy(
      Uint8List src, int srcPos, Uint8List dest, int destPos, int length) {
    for (int i = srcPos; i < srcPos + length; i++) {
      dest[destPos++] = src[i];
    }
  }

  static Uint8List doubleDigest(Uint8List input) {
    var digest = sha256.convert(input);
    var dDigeset = sha256.convert(digest.bytes);

    return dDigeset.bytes;
  }

  static Uint8List halfSha512(Uint8List bytes) {
    var s512 = sha512.convert(bytes);
    return s512.bytes.sublist(0, 32);
  }

  static Uint8List addList(Uint8List left, Uint8List right) {
    Uint8List buffer = new Uint8List(left.length + right.length);
    for (int i = 0; i < left.length; i++) {
      buffer[i] = left[i];
    }

    for (int j = 0; j < right.length; j++) {
      buffer[left.length + j] = right[j];
    }

    return buffer;
  }

  static Uint8List encodeToBytesChecked(Uint8List input, Uint8List version) {
    Uint8List buffer = addList(version, input);

    var dRes = doubleDigest(buffer);

    var checkSum = dRes.sublist(0, 4);

    var output = addList(buffer, checkSum);

    return output;
  }

  static String encodeToBytes(Uint8List input) {
    Base58Codec codec = const Base58Codec(rippleAlphabet);
    var encodeStr = codec.encode(input);
    return encodeStr;
  }

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Format //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

  static String formatAsTruncated(String str) {
    if (str.length > 26) {
      return str.substring(0, 26) + "[...]";
    } else if (str.length == 0) {
      return "(empty string)";
    } else {
      return str;
    }
  }

  static String formatAsHumanSize(num size) {
    if (size < 1024) return "$size B";
    if (size < 1024 * 1024) return "${_format(size / 1024)} KB";
    if (size < 1024 * 1024 * 1024) return "${_format(size / (1024 * 1024))} MB";
    return "${_format(size / (1024 * 1024 * 1024))} GB";
  }

  static String formatBytesAsHexString(Uint8List bytes) {
    var result = new StringBuffer();
    for (var i = 0; i < bytes.lengthInBytes; i++) {
      var part = bytes[i];
      result.write('${part < 16 ? '0' : ''}${part.toRadixString(16)}');
    }

    var lowerStr = result.toString();
    return lowerStr.toUpperCase();
  }

  static String _format(double val) {
    if (val.isInfinite) {
      return "INF";
    } else if (val.isNaN) {
      return "NaN";
    } else {
      return val.floor().toString() +
          "." +
          (100 * (val - val.toInt())).toInt().toString();
    }
  }

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Data ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
  static Uint8List createUint8ListFromBigInt(BigInt big) {
    var hexStr = big.toRadixString(16);
    return HEX.decode(hexStr);
  }

  static BigInt createBigIntFromUint8List(Uint8List bytes) {
    var hexStr = HEX.encode(bytes);
    return BigInt.parse(hexStr, radix: 16);
  }

  static Uint8List createUint8ListFromString(String s) {
    var ret = new Uint8List(s.length);
    for (var i = 0; i < s.length; i++) {
      ret[i] = s.codeUnitAt(i);
    }
    return ret;
  }

  static Uint8List createUint8ListFromHexString(String hex) {
    var result = new Uint8List(hex.length ~/ 2);
    for (var i = 0; i < hex.length; i += 2) {
      var num = hex.substring(i, i + 2);
      var byte = int.parse(num, radix: 16);
      result[i ~/ 2] = byte;
    }
    return result;
  }

  static Uint8List createUint8ListFromSequentialNumbers(int len) {
    var ret = new Uint8List(len);
    for (var i = 0; i < len; i++) {
      ret[i] = i;
    }
    return ret;
  }

  /**
   * Get random byte array.
   * @param length Random array length.
   * @return Byte array.
   */
  static Uint8List getRandomBytes(int length) {
    Uint8List bytes = new Uint8List(length);
    Random r = new Random();

    for (int i = 0; i < 16; i++) {
      int rInt = r.nextInt(255);
      bytes[i] = rInt;
    }

    return bytes;
  }

  static Uint8List paddingPass(Uint8List password, int keyLength) {
    if (password.length == keyLength) return password;
    Uint8List retByte = new Uint8List(keyLength);
    if (password.length < keyLength) {
      int byteToPad = keyLength - password.length;
      for (int i = 0; i < keyLength; i++) {
        if (i < password.length)
          retByte[i] = password[i];
        else
          retByte[i] = byteToPad;
      }
    } else {
      for (int i = 0; i < keyLength; i++) {
        retByte[i] = password[i];
      }
    }
    return retByte;
  }
}

==> dart_chainsql_api-1.0.3.tar.gz: lib/src/utils/Util.dart <==
// Copyright (c) 2013-present, the authors of the Pointy Castle project
// This library is dually licensed under LGPL 3 and MPL 2.0.
// See file LICENSE for more information.

//library pointycastle.test.test.src.helpers;

import "dart:typed_data";
import 'dart:math';

import 'package:crypto/crypto.dart';

import "package:base58check/base58.dart";
import "package:hex/hex.dart";
import 'package:pointycastle/api.dart';

import 'B58IdentiferCodecs.dart';

class Util {
  static const String rippleAlphabet =
      "zpxhncf39wBUDNEGHJKLM4PQRST7VWXYZ2badeCg65jkm8oFqi1tuvAsyr";

  ///
  ///  @param      src      the source array.
  ///  @param      srcPos   starting position in the source array.
  ///  @param      dest     the destination array.
  ///  @param      destPos  starting position in the destination data.
  ///  @param      length   the number of array elements to be copied.
  ///  @exception  IndexOutOfBoundsException  if copying would cause
  ///                access of data outside array bounds.
  ///  @exception  ArrayStoreException  if an element in the <code>src</code>
  ///                array could not be stored into the <code>dest</code> array
  ///                because of a type mismatch.
  ///  @exception  NullPointerException if either <code>src</code> or
  ///                <code>dest</code> is <code>null</code>.
  ///
  static arraycopy(
      Uint8List src, int srcPos, Uint8List dest, int destPos, int length) {
    for (int i = srcPos; i < srcPos + length; i++) {
      dest[destPos++] = src[i];
    }
  }

  static Uint8List doubleDigest(Uint8List input) {
    var digest = sha256.convert(input);
    var dDigeset = sha256.convert(digest.bytes);

    return dDigeset.bytes;
  }

  static Uint8List halfSha512(Uint8List bytes) {
    var s512 = sha512.convert(bytes);
    return s512.bytes.sublist(0, 32);
  }

  static Uint8List addList(Uint8List left, Uint8List right) {
    Uint8List buffer = new Uint8List(left.length + right.length);
    for (int i = 0; i < left.length; i++) {
      buffer[i] = left[i];
    }

    for (int j = 0; j < right.length; j++) {
      buffer[left.length + j] = right[j];
    }

    return buffer;
  }

  static Uint8List encodeToBytesChecked(Uint8List input, Uint8List version) {
    Uint8List buffer = addList(version, input);

    var dRes = doubleDigest(buffer);

    var checkSum = dRes.sublist(0, 4);

    var output = addList(buffer, checkSum);

    return output;
  }

  static String encodeToBytes(Uint8List input) {
    Base58Codec codec = const Base58Codec(rippleAlphabet);
    var encodeStr = codec.encode(input);
    return encodeStr;
  }

  static Uint8List getPubBytesFromString(String publicKey) {
    Uint8List pubBytes = null;
    if (publicKey.length == 66) {
      pubBytes = Util.createUint8ListFromHexString(publicKey);
    } else {
      pubBytes = B58IdentiferCodecs.decode(
          publicKey, B58IdentiferCodecs.VER_ACCOUNT_PUBLIC);
    }
    return pubBytes;
  }

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Format //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

  static String formatAsTruncated(String str) {
    if (str.length > 26) {
      return str.substring(0, 26) + "[...]";
    } else if (str.isEmpty) {
      return "(empty string)";
    } else {
      return str;
    }
  }

  static String formatAsHumanSize(num size) {
    if (size < 1024) return "$size B";
    if (size < 1024 * 1024) return "${_format(size / 1024)} KB";
    if (size < 1024 * 1024 * 1024) return "${_format(size / (1024 * 1024))} MB";
    return "${_format(size / (1024 * 1024 * 1024))} GB";
  }

  static String formatBytesAsHexString(Uint8List bytes) {
    var result = new StringBuffer();
    for (var i = 0; i < bytes.lengthInBytes; i++) {
      var part = bytes[i];
      result.write('${part < 16 ? '0' : ''}${part.toRadixString(16)}');
    }

    var lowerStr = result.toString();
    return lowerStr.toUpperCase();
  }

  static String formatHexStringFromString(String str) {
    var bytes = createUint8ListFromString(str);
    return formatBytesAsHexString(bytes);
  }

  static String _format(double val) {
    if (val.isInfinite) {
      return "INF";
    } else if (val.isNaN) {
      return "NaN";
    } else {
      return val.floor().toString() +
          "." +
          (100 * (val - val.toInt())).toInt().toString();
    }
  }

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Data ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
  static Uint8List createUint8ListFromBigInt(BigInt big) {
    var hexStr = big.toRadixString(16);
    return HEX.decode(hexStr);
  }

  static BigInt createBigIntFromUint8List(Uint8List bytes) {
    var hexStr = HEX.encode(bytes);
    return BigInt.parse(hexStr, radix: 16);
  }

  static Uint8List createUint8ListFromString(String s) {
    var ret = new Uint8List(s.length);
    for (var i = 0; i < s.length; i++) {
      ret[i] = s.codeUnitAt(i);
    }
    return ret;
  }

  static Uint8List createUint8ListFromHexString(String hex) {
    var result = new Uint8List(hex.length ~/ 2);
    for (var i = 0; i < hex.length; i += 2) {
      var num = hex.substring(i, i + 2);
      var byte = int.parse(num, radix: 16);
      result[i ~/ 2] = byte;
    }
    return result;
  }

  static Uint8List createUint8ListFromSequentialNumbers(int len) {
    var ret = new Uint8List(len);
    for (var i = 0; i < len; i++) {
      ret[i] = i;
    }
    return ret;
  }

  ///
  /// Get random byte array.
  /// @param length Random array length.
  /// @return Byte array.
  ///
  static Uint8List getRandomBytes(int length) {
    Uint8List bytes = new Uint8List(length);
    Random r = new Random();

    for (int i = 0; i < 16; i++) {
      int rInt = r.nextInt(255);
      bytes[i] = rInt;
    }

    return bytes;
  }

  static Uint8List paddingPass(Uint8List password, int keyLength) {
    if (password.length == keyLength) return password;
    Uint8List retByte = new Uint8List(keyLength);
    if (password.length < keyLength) {
      int byteToPad = keyLength - password.length;
      for (int i = 0; i < keyLength; i++) {
        if (i < password.length) {
          retByte[i] = password[i];
        } else {
          retByte[i] = byteToPad;
        }
      }
    } else {
      for (int i = 0; i < keyLength; i++) {
        retByte[i] = password[i];
      }
    }
    return retByte;
  }

  static Uint8List hmac(Uint8List key, Uint8List bytes) {
    var mac = new Mac('SHA-256/HMAC');
    mac.init(new KeyParameter(key));
    mac.update(bytes, 0, bytes.length);
    final int MACBYTELENGTH = 32;
    Uint8List macBytes = new Uint8List(MACBYTELENGTH);
    mac.doFinal(macBytes, 0);
    return macBytes;
  }
}
